﻿; SendKeyWindow.ahk[UTF-8] (2025/01/28 11:00)
; for AutoHotKey v2.0.19 or later
;
; 指定のウィンドウにキーを送る
;    SendKeyWindow KeyCodes Target (ArrowStyle) (DenyStyle) (DispError) (Debug)
;       KeyCodes
;          (キー送出モード : XXX_YYY)
;            修飾キー + "_" + 送出キーを送る ※修飾キーのみを送ることは出来ないのでキーをAltなどのキーワード
;            指定すること (http://ahkscript.github.io/ja/docs/v2/KeyList.htm) 区切りの "_"必須
;            修飾キー：SHIFT=S, CTRL=C, ALT=A 順不同で大文字小文字区別なし
;            送出キー：大文字小文字区別無し、AutoHotKeyのキー指定で{}内に記述するキー名
;              ex) CSA_R : CTRL+SHIFT+ALT+R を送る
;              ex) C_C   : CTRL+C を送る
;              ex) _A    : A を送る
;          (PostMessageモード : "iMessage,iWParam,iLParam")
;            両端を "" で囲みカンマ区切りで値を指定する
;            iMessage : WM_SYSCOMMAND などウィンドウメッセージの数値を指定(必須)
;                       ※WM_SYSCOMMANDのみ特例で文字列記述WM_SYSCOMMANDで0x112と解釈されます
;            iWParam :  SC_MINIMIZE などＷパラメータの数値を指定(省略可ですが殆どのWMで必須だったかと)
;            iLParam :  Ｗパラメータに付随するＬパラメータの数値を指定(省略可)
;       Target
;          ウィンドウ文字列/ahk_exe フルパス/ahk_class クラスID など両端""でAutoHotKey
;          独自のウィンドウ指定記述、現在アクティブなウィンドウを指定する場合は"A"
;          https://ahkscript.github.io/ja/docs/v2/misc/WinTitle.htm
;          対象ウィンドウのウィンドウスタイルは WS_VISIBLE ビットが有効で WS_DISABLED ビットが無効の
;          ものとしています。複数ウィンドウに該当する場合はエラー表示し処理しません。
;          マッチを正規表現で記述したい時は先頭に"REGEX:"を書きます
;          AutoHotKeyV2の正規表現文法は下記参照
;          https://ahkscript.github.io/ja/docs/v2/misc/RegEx-QuickRef.htm
;          ex) "ahk_exe C:\Program Files\Mozilla Firefox\firefox.exe"
;          ex) "ahk_exe C:\Program Files\Vivaldi\Application\vivaldi.exe ahk_class Chrome_WidgetWin_1"
;       ArrowStyle
;          処理対象であるウィンドウスタイルのビット和(andをとって0以外になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x10000000(WS_VISIBLE) が指定されたものとします
;          全てを対象外にしてしまう 0x00000000 は指定できません
;       DenyStyle
;          処理対象から除外するウィンドウスタイルのビット和(andをとって0になれば対象)の16進表記
;          省略された場合やdef/defaultと書かれた時は 0x08000000(WS_DISABLED) が指定されたものとします
;          全てを対象外にしてしまう 0xFFFFFFFF は指定できません
;       DispError
;          1/on/disp : 対象ウィンドウが見つからない時や複数ヒットした際のエラーダイアログを出す
;          [any] : 上記以外が指定されるか無指定ならエラー時にダイアログを出さない
;       DebugMode
;          1/on/debug : デバッグモードON
;          [any] : 上記以外が指定されるか無指定ならデバッグモードOFF
;

/* 同時に複数実行を許可する */
#SingleInstance Off

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておかないと仮想デスクトップで動作しない */
DetectHiddenWindows True

WM_SYSCOMMAND := 0x112
SC_MINIMIZE := 0xF020
SC_MAXIMIZE := 0xF030
SC_RESTORE := 0xF120

/* ウィンドウスタイル定義 */
WS_POPUP        := 0x80000000
WS_CHILD        := 0x40000000
WS_MINIMIZE     := 0x20000000
WS_VISIBLE      := 0x10000000
WS_DISABLED     := 0x08000000
WS_CLIPSIBLINGS := 0x04000000
WS_CLIPCHILDREN := 0x02000000
WS_MAXIMIZE     := 0x01000000
WS_BORDER       := 0x00800000
WS_DLGFRAME     := 0x00400000
WS_VSCROLL      := 0x00200000
WS_HSCROLL      := 0x00100000
WS_SYSMENU      := 0x00080000
WS_SIZEBOX      := 0x00040000
WS_MINIMIZEBOX  := 0x00020000
WS_MAXIMIZEBOX  := 0x00010000
WS_OVERLAPPED   := 0x00000000

;// Firefox ESRの例    style=0x16CF0000
;// Firefox Betaの例   style=0x16C70000
;// Chrmoe の例        style=0x16CF0000
;// Chrome最大化の例   style=0x17CF0000
;// Taskmgrの例        style=0x14CF0000
;// WindowSpyの例      style=0x14CE0000
;// ポップアップ系の例 style=0x94CA0000



/* コマンドオプション数チェック */
if( A_Args.Length < 2 ){
	DispUsage()
}

/* モードとウィンドウ指定 */
strKeys := A_Args[1]
strTarget := A_Args[2]
arg_regx := 2
if( StrUpper(SubStr(strTarget,1,6)) = "REGEX:" ){
	arg_regx := "RegEx"
	strTarget := SubStr(strTarget,7)
}
UniqueID := 0
iDisp := 0
iDebug := 0
iArrowStyle := 0
iDenyStyle := 0xFFFFFFFF


/* 必須ウィンドウスタイルの扱い */
if( A_Args.Length >= 3 ){
	iArrowStyle := A_Args[3]
}
if( iArrowStyle = 0 or StrUpper(iArrowStyle) = "DEF" or StrUpper(iArrowStyle) = "DEFAULT" ){
	iArrowStyle := WS_VISIBLE
}else{
	if( IsNumber(iArrowStyle) ){
		iArrowStyle := Number(iArrowStyle)
	}
}

/* 除外ウィンドウスタイルの扱い */
if( A_Args.Length >= 4 ){
	iDenyStyle := A_Args[4]
}
if( iDenyStyle = 0xFFFFFFFF or StrUpper(iDenyStyle) = "DEF" or StrUpper(iDenyStyle) = "DEFAULT" ){
	iDenyStyle := WS_DISABLED
}else{
	if( IsNumber(iDenyStyle) ){
		iDenyStyle := Number(iDenyStyle)
	}
}


/* エラーダイアログの扱い */
if( A_Args.Length >= 5 ){
	iDisp := A_Args[5]
}
if( iDisp = 1 or StrUpper(iDisp) = "ON" or StrUpper(iDisp) = "DISP" ){
	iDisp := 1
}else{
	iDisp := 0
}

/* デバッグモードの指定 */
if( A_Args.Length >= 6 ){
	iDebug := A_Args[6]
}
if( iDebug = 1 or StrUpper(iDebug) = "ON" or StrUpper(iDebug) = "DEBUG" ){
	iDebug := 1
}else{
	iDebug := 0
}


/* タイトルマッチモードの設定(2=部分一致, "RegEx"=正規表現) */
SetTitleMatchMode(arg_regx)

/* フルパスに合致するウィンドウを列挙(非表示ウィンドウも対象としているため沢山ヒットする) */
listWin := WinGetList(strTarget)
idxWin := 0
findCount := 0

/* ウィンドウスタイルでマスク */
for hWin in listWin{
	idxWin := idxWin + 1
	try{
		style := WinGetStyle("ahk_id " hWin)
		if( style & iArrowStyle and not(style & iDenyStyle) ){
			findCount := findCount + 1
			UniqueID := hWin
		}
	}
}

if( UniqueID = 0 ){
	ExitApp(1)
}

if( findCount != 1 ){
	if( iDisp or iDebug ){
		MsgBox(strTarget " has multiple windows.")
	}
	ExitApp(1)
}

/* 修飾キーと送出キーの解釈 */
if( InStr(strKeys, "_") ){

	/* 修飾キー_送出キー モード */
	aryKeys := StrSplit(StrLower(strKeys),"_")
	if( aryKeys.Length = 2 ){

		strExtKey := aryKeys[1]
		strSndKey := aryKeys[2]
		strTemp := ""

		if( InStr(strExtKey,"c") != 0 ){
			strTemp := strTemp "^"
		}
		if( InStr(strExtKey,"s") != 0 ){
			strTemp := strTemp "+"
		}
		if( InStr(strExtKey,"a") != 0 ){
			strTemp := strTemp "!"
		}
		strTemp := strTemp "{" strSndKey "}"

		if( iDebug ){
			MsgBox("strTemp=" strTemp)
		}

		try{
			WinActivate("ahk_id " UniqueID)
			Send(strTemp)
		}catch TargetError as err{
			if( iDisp or iDebug ){
				MsgBox("WinActivate Error.")
			}
			ExitApp(1)
		}
	}else{
		DispUsage()
	}

}else{

	/* PostMessageモード */
	strKeys := StrReplace(strKeys, ' ', "")
	strKeys := StrReplace(strKeys, '"', "")
	aryTemp := StrSplit(strKeys, ",")

	;MsgBox( aryTemp.Length )
	aryTemp[1] := StrReplace(aryTemp[1], "WM_SYSCOMMAND", "0x112")

	try{
		if( aryTemp.Length = 1 ){
			PostMessage( Number(aryTemp[1]),,,, "ahk_id " UniqueID)
		}else if( aryTemp.Length = 2 ){
			PostMessage( Number(aryTemp[1]), Number(aryTemp[2]),,, "ahk_id " UniqueID)
		}else if( aryTemp.Length = 3 ){
			PostMessage( Number(aryTemp[1]), Number(aryTemp[2]), Number(aryTemp[3]),, "ahk_id " UniqueID)
		}else{
			DispUsage()
		}
	}catch{
		DispUsage()
	}
}

ExitApp(0)

DispUsage(){
	msg := 'Usage:`n'
	msg := msg 'SendKeyWindow.exe KeyCodes Target (...options)`n`n'
	msg := msg '[KeyCodes]`n'
	msg := msg '    ex) CSA_R = CTRL+SHIFT+ALT+R`n'
	msg := msg '    ex) CS_T = CTRL+SHIFT+T`n'
	msg := msg '    ex) C_A = CTRL+A`n'
	msg := msg '    ex) _X = X`n'
	msg := msg '    ex) "0x112,0xF020,0x0000" = PostMesage(iMsg,iWParam,iLParam)`n'
	msg := msg '[Target]`n'
	msg := msg '   "window-title / REGEX:match-title-str`n'
	msg := msg '    ahk_exe full-path`n'
	msg := msg '    ahk_class class-name..."`n'
	msg := msg '[ArrowStyle]`n'
	msg := msg '    enabled WindowStyle Bits`n'
	msg := msg '    def/default = 0x10000000(WS_VISIBLE)`n'
	msg := msg '    ex) 0x16C70000`n'
	msg := msg '[DenyStyle]`n'
	msg := msg '    disabled WindowStyleBits`n'
	msg := msg '    def/default = 0x08000000(WS_DISABLED)`n'
	msg := msg '    ex) 0xE8000000`n'
	msg := msg '[DispError]`n'
	msg := msg '    1/on/disp = display ErrorDialog`n'
	msg := msg '[DebugMode]`n'
	msg := msg '    1/on/debug = Debug mode`n'
	MsgBox(msg)
	ExitApp(1)
}


